/**************************************************************************** 
*  Copyright 2007 C&H Technologies, Inc.  All Rights Reserved.					    * 
*****************************************************************************/
/****************************************************************************
*
*	ma200_ansi.c
*
*	ANSI-C source module for the MA200 Apeture A/D M-Module.
*
* This file contains high level functions to control a MA200 Apeture A/D
*	M-Module from C&H Technologies, Inc.  This file is written entirely in 
*	ANSI-C so that is may be compiled in any environment with a supporting 
*	ANIS-C compiler. It uses the APIS library to perform instrument 
*	communications.  
*
*	This file is organized such that modifications are easy.  All functions 
*	that might need to be modified are located at the end of this file.  
*	Comments are provided that explain tasks of the function and how it might 
*	be modified.  All other function should not need to be modified.
*
*	C&H Part Number: 11027869
*
*	Revision History
*	-	Original Release Version 4.0 - All previos versions where legacy VXIpnp 
*		drivers.  See the legacy VXIpnp source code for details on previous 
*		versions.
*****************************************************************************/
#include <ansi_c.h>
#include "ma200_ansi.h"


/**************************************************************************** 
*  Non Exported Defines
*****************************************************************************/
/*  Driver Revision  */
#define		MA200_DRIVER_REV			"4.0"

/*  Input Mode Defines  */
#define		MA200_BIPOLAR_10VPP_INPUT			0
#define		MA200_BIPOLAR_20VPP_INPUT			1
#define		MA200_UNIPOLAR_10VPP_INPUT		2
#define		MA200_UNIPOLAR_20VPP_INPUT		3


/**************************************************************************** 
*  Error Checking Macros
*****************************************************************************/
#ifndef checkParamRange
#define  checkParamRange(value, min, max, param)		if ((value > max) || (value < min)) { \
																											error = MA200_ERROR_PARAMETER1 + param-1;\
																											goto Error; \
																										} else 
#endif																				
																				
#ifndef checkParamFloatRange
#define  checkParamFloatRange(value, min, max, param)		if ((FP_Compare (value, max) == 1) || (FP_Compare (value, min) == -1)) { \
																													error = MA200_ERROR_PARAMETER1 + param-1;\
																													goto Error; \
																												} else 
#endif																				

#ifndef checkParamPointer
#define  checkParamPointer(pointer, param)		if (pointer == NULL) { \
																							error = MA200_ERROR_PARAMETER1 + param-1;\
																							goto Error; \
																							} else 
#endif																				

																				
#ifndef checkErr
#define checkErr(fCall)      if (error = (fCall), (error = (error < 0) ? error : MA200_SUCCESS)) \
                                 {goto Error;}  else
#endif

#ifndef apisCheckErr
#define apisCheckErr(fCall)      if (error = (fCall), (error = (error != 0) ? error : MA200_SUCCESS)) \
                                 {	handle->apis_error = error;\
																		error = MA200_ERROR_APIS; \
																		goto Error;}  else
#endif

#ifndef apisCheckErrInit
#define apisCheckErrInit(fCall)      if (error = (fCall), (error = (error != 0) ? error : MA200_SUCCESS)) \
                                 {	(*handle)->apis_error = error;\
																		error = MA200_ERROR_APIS; \
																		goto Error;}  else
#endif


/**************************************************************************** 
*  Defined Structures
*****************************************************************************
/*  None  */

/**************************************************************************** 
*  Constants
*****************************************************************************/

static const char *errorMessageTable[MA200_SPECIFIC_ERROR_END - MA200_SPECIFIC_ERROR_BASE] = 
						{	"The value passed to the function has been reserved for future use.",
							"This function cannot be performed while the instrument is running.",
							"Fifo empty. No valid data was found in the fifo.",
							"The input mode attribute has not been initialized.  You must call the function ma200_SetInputModeAndRange()\
before using either ma200_ReadFifo() or ma200_SetAperture().",
							"An invalid parameter was passed to the function in parameter 1",
							"An invalid parameter was passed to the function in parameter 2",
							"An invalid parameter was passed to the function in parameter 3",
							"An invalid parameter was passed to the function in parameter 4",
							"The module returned an invalid ID.",
							"An error occurred in the APIS Platform Support Module.  Use the function ma204_GetApisError() to retrieve the APIS error code.",
							"The operation could not be completed because the system is out of memory."
						};

static const char *warningMessageTable[MA200_SPECIFIC_WARNING_END - MA200_SPECIFIC_WARNING_BASE] =
						{	"There were less samples stored in the FIFO than what was requested."
						};

/**************************************************************************** 
*  Non Exported Functions
*****************************************************************************/
static Int32 ma200_IDWriteByte(MA200_HANDLE handle, UInt8 value);
static Int32 ma200_IDWriteBit(MA200_HANDLE handle,  Bool value);
static Int32 ma200_IDReadByte(MA200_HANDLE handle, UInt8 *value);
static Int32 ma200_IDReadBit(MA200_HANDLE handle,  Bool *value);



/*****************************************************************************
*	ma200_RevisionQuery
*
*	Returns the version number of this driver.  
******************************************************************************/
Int32 ma200_RevisionQuery (Int8 driver_rev[])
{	Int32 error = MA200_SUCCESS;

/*  Check parameters  */
	checkParamPointer(driver_rev, 2);
	
	sprintf(driver_rev, "%s", MA200_DRIVER_REV);
Error:
	return error;
}



/*****************************************************************************
*	ma200_ErrorMessage
*
*	Returns a text description of the specified status code.  
******************************************************************************/
Int32 ma200_ErrorMessage (Int32 statusCode, Int8 errorMessage[])
{	Int32 error = MA200_SUCCESS;
	
/*  Check parameters  */
	checkParamPointer(errorMessage, 3);

/*  Check driver defined error codes  */
	if((statusCode >= MA200_SPECIFIC_ERROR_BASE) && (statusCode < MA200_SPECIFIC_ERROR_END))
	{	strcpy(errorMessage, errorMessageTable[statusCode - MA200_SPECIFIC_ERROR_BASE]);
	}
	else if((statusCode >= MA200_SPECIFIC_WARNING_BASE) && (statusCode < MA200_SPECIFIC_WARNING_END))
	{	strcpy(errorMessage, warningMessageTable[statusCode - MA200_SPECIFIC_WARNING_BASE]);
	}
	else if (statusCode == 0)
		sprintf(errorMessage, "No Error Occured");
	else
		sprintf(errorMessage, "Unknown Error Code [0x%x]", statusCode);
		
Error:
	return error;
}


/*****************************************************************************
*	ma200_GetIdentWord
*
*	Reads the specified offset from the module's ID space and returns
*	the result.
******************************************************************************/
Int32 ma200_GetIdentWord (MA200_HANDLE handle, UInt16 offset, UInt16 *value)
{	Int32 error = MA200_SUCCESS;
	UInt8 data;

/*  Check parameters  */
	checkParamPointer(value, 3);

	*value = 0;
	
/*  Send the read command to the serial EEPROM  */	
	checkErr(ma200_IDWriteByte(handle, 0x80 | offset));
	
/*  Read upper byte  */	
	checkErr(ma200_IDReadByte(handle, &data));
	*value = data << 8;
	
/*  Read lower byte  */	
	checkErr(ma200_IDReadByte(handle, &data));
	*value |= data;
	
Error:	
	ma200_WriteRegister(handle, MA200_ID_ACCESS_REGISTER, 0);
	return error;
}

										
/*****************************************************************************
*	ma200_ConfigureSampleClock
*
*	Configures the clock the the ma200 uses for sampling.  Sets up the 
*	clock control register according to values specified in the parameters.
******************************************************************************/
Int32 ma200_ConfigureSampleClock (MA200_HANDLE handle, Int32 clockSource, Int32 clockPrescaler)
{	Int32 error = MA200_SUCCESS;
	UInt16 value = 0;

/*  Check parameters  */
	checkParamRange(clockPrescaler, 0, 3, 3);
	checkParamRange(clockSource, 0, 7, 2);

/*  Get current value of Status/Control Register  */	
	checkErr(ma200_ReadRegister(handle, MA200_CONTROL_STATUS_REGISTER, &value));

/*  Form new register value and write it  */
	value &= ~(MA200_CLKSEL_BITS | MA200_PSC_BITS | MA200_STATUS_BITS);	/* Do not write to status bits so
																																						   we don't clear interrupts  */
	value |= ((clockSource << 8) | (clockPrescaler << 11)); 									 
	checkErr(ma200_WriteRegister(handle, MA200_CONTROL_STATUS_REGISTER, value));
	
Error:
	return error;
}


/*****************************************************************************
*	ma200_ConfigureRunSource
*	
*	Sets the source of the signal that the ma200 uses to run and stop.
******************************************************************************/
Int32 ma200_ConfigureRunSource (MA200_HANDLE handle, Int32 runSource)
{	Int32 error = MA200_SUCCESS;
	UInt16 value = 0;

	checkParamRange(runSource, 0, 7, 2);

	
	if(runSource == 1)
	{ error = MA200_ERROR_RESERVED_VALUE;
		goto Error;
	}

/*  Read the current value of the control/status register  */	
	checkErr(ma200_ReadRegister(handle, MA200_CONTROL_STATUS_REGISTER, &value));

/*  Form new value and write the register  */	
	value &= ~(MA200_RUN_SEL_BITS | MA200_STATUS_BITS);  									/* Do not write to status bits so
																																						   we don't clear interrupts  */
	value |= runSource << 4;
	checkErr(ma200_WriteRegister(handle, MA200_CONTROL_STATUS_REGISTER, value));
	
Error:
	return error;
}


/*****************************************************************************
*	ma200_StoreAll
*	
*	Sets or clears the store all bit in the control status register
******************************************************************************/
Int32 ma200_StoreAll (MA200_HANDLE handle,  Bool storeAll)
{	Int32 error = MA200_SUCCESS;
	UInt16 value = 0;

/*  Check parameters  */
	checkParamRange(storeAll, 0, 1, 2);

/*  Read the current value of the control/status register  */	
	checkErr(ma200_ReadRegister(handle, MA200_CONTROL_STATUS_REGISTER, &value));

/*  Form new value and write the register  */	
	value &= ~(MA200_STORE_ALL_BIT | MA200_STATUS_BITS); 									/* Do not write to status bits so
																																						   we don't clear interrupts  */
	if(storeAll == MA200_TRUE)
		value |= MA200_STORE_ALL_BIT;
	checkErr(ma200_WriteRegister(handle, MA200_CONTROL_STATUS_REGISTER, value));

Error:
	return error;
}


/*****************************************************************************
*	ma200_EnableInterrupts 
*
*	Configures the ma200 to produce interrupts.  This function does not
*	configure the conditions on which interrupts occur.
******************************************************************************/
Int32 ma200_EnableInterrupts (MA200_HANDLE handle,  Bool dataStored,  Bool fifoFull,  Bool fifoHalfFull)
{	Int32 error = MA200_SUCCESS;
	UInt16 value = 0;
	
/*  Check parameters  */
	checkParamRange(dataStored, 0, 1, 2);
	checkParamRange(fifoFull, 0, 1, 3);
	checkParamRange(fifoHalfFull, 0, 1, 4);


/*  Configure the Ma203  */	
	checkErr(ma200_ReadRegister(handle, MA200_INTERRUPT_CONTROL_REGISTER, &value));
	value &= ~MA200_INTERRUPT_ENABLE_BITS;
	value |= ((dataStored << 15) | (fifoFull << 14) | (fifoHalfFull << 13));
	checkErr(ma200_WriteRegister(handle, MA200_INTERRUPT_CONTROL_REGISTER, value));

Error:
	return error;
}



/*****************************************************************************
*	ma200_ClearInterrupts
*
*	Clears interrupt pendings for the channels specified by the
*	clearInterruptMask parameter.
******************************************************************************/
Int32 ma200_ClearInterrupts (MA200_HANDLE handle,  Bool dataStored,  Bool fifoFull,  Bool fifoHalfFull)
{	Int32 error = MA200_SUCCESS;
	UInt16 value;

/*  Check parameters  */
	checkParamRange(dataStored, 0, 1, 2);
	checkParamRange(fifoFull, 0, 1, 3);
	checkParamRange(fifoHalfFull, 0, 1, 4);

/*  Read current value of the control/status register  */
	checkErr(ma200_ReadRegister(handle, MA200_CONTROL_STATUS_REGISTER, &value));
	
/*  Set bits to clear interrupts  */	
	value &= ~MA200_STATUS_BITS;
	value |= ((dataStored << 15) | (fifoFull << 14) | (fifoHalfFull << 13));
	
/*  Write new value to the register  */	
	checkErr(ma200_WriteRegister(handle, MA200_CONTROL_STATUS_REGISTER, value));

Error:
	return error;
}


/*****************************************************************************
*	ma200_GetInterruptPending
*
*	Returns the value of the interrupt pending register
******************************************************************************/
Int32 ma200_GetInterruptPending (MA200_HANDLE handle, Bool *dataStored, Bool *fifoFull, Bool *fifoHalfFull)
{	Int32 error = MA200_SUCCESS;
	UInt16 value;
	
/*  Check parameters  */
	checkParamPointer(dataStored, 2);
	checkParamPointer(fifoHalfFull, 3);
	checkParamPointer(fifoFull, 4);
	
	*dataStored = 0;
	*fifoFull = 0;
	*fifoHalfFull = 0;
	
/*  Read control status register  */
	checkErr(ma200_ReadRegister(handle, MA200_CONTROL_STATUS_REGISTER, &value));
	
/*  Check flag bits  */	
	if(value & MA200_DATA_STORED_BIT)
		*dataStored = 1;
	if(value & MA200_FIFO_FULL_BIT)
		*fifoFull = 1;
	if(value & MA200_FIFO_HALF_FULL_BIT)
		*fifoHalfFull = 1;
Error:
	return error;
}




/*****************************************************************************
*	ma200_Run
*
*	Sets the run bit in the control status register.  This will begin sampling.
******************************************************************************/
Int32 ma200_Run (MA200_HANDLE handle)
{	Int32 error = MA200_SUCCESS;
	UInt16 value;

/*  Read current value of the control/status register  */	
	checkErr(ma200_ReadRegister(handle, MA200_CONTROL_STATUS_REGISTER, &value));
	
/*  Form new value  */	
	value &= ~MA200_STATUS_BITS;			/* 	Do not write to status bits so																																							
																					we don't clear interrupts  */	
	value |= MA200_RUN_BIT;
	
/*  Write the new value to the register  */	
	checkErr(ma200_WriteRegister(handle, MA200_CONTROL_STATUS_REGISTER, value));
Error:
	return error;
}


/*****************************************************************************
*	ma200_Stop
*
*	Clears the run bit inthe contorl status register.  This will stop sampling.
******************************************************************************/
Int32 ma200_Stop (MA200_HANDLE handle)
{	Int32 error = MA200_SUCCESS;
	UInt16 value;

/*  Read current value of the control/status register  */
	checkErr(ma200_ReadRegister(handle, MA200_CONTROL_STATUS_REGISTER, &value));

/*  Form new value  */
	value &= ~(MA200_RUN_BIT | MA200_STATUS_BITS);			/* 	Do not write to status bits so																																							
																									  					we don't clear interrupts  */	
																									  					
/*  Write new value to the control/status register  */																									  					
	checkErr(ma200_WriteRegister(handle, MA200_CONTROL_STATUS_REGISTER, value));
	
Error:
	return error;
}


/*****************************************************************************
*	ma200_SingleSample
*
*	Set the sample bit in the control status register.  This forces one
*	sample and store to occur.  If this function is called while the
*	run bit is set it will return an error.
******************************************************************************/
Int32 ma200_SingleSample (MA200_HANDLE handle)
{	Int32 error = MA200_SUCCESS;
	UInt16 value;
	
/*  Read current value of the control/status register  */
	checkErr(ma200_ReadRegister(handle, MA200_CONTROL_STATUS_REGISTER, &value));

/*  Make sure the MA200 is not running  */	
	if(value & MA200_RUN_BIT)
	{	error = MA200_ERROR_RUNNING;
		goto Error;
	}
	else
	{	value &=  ~MA200_STATUS_BITS;		/* 	Do not write to status bits so																																							
																					we don't clear interrupts  */	
																					
	/*  Set the single sample bit and write tot he register  */																					
		value |= MA200_CNV_BIT;
		checkErr(ma200_WriteRegister(handle, MA200_CONTROL_STATUS_REGISTER, value));
	}
																																																										
Error:
	return error;
}


/*****************************************************************************
*	ma200_LastStoredValue
*
*	Returns the value of the last value stored register
******************************************************************************/
Int32 ma200_LastStoredValue (MA200_HANDLE handle, UInt16 *lastStoredValue)
{	Int32 error = MA200_SUCCESS;

/*  Check parameters  */
	checkParamPointer(lastStoredValue, 2);
	
	checkErr(ma200_ReadRegister(handle, MA200_LAST_VALUE_STORED_REGISTER, lastStoredValue));
Error:
	return error;
}


/*****************************************************************************
*	ma200_ReadFifo
*
*	Reads the next time stamp/data pair from the fifo.  If no valid data
*	exists in the fifo this function will return an error.
******************************************************************************/
Int32 ma200_ReadFifo (MA200_HANDLE handle, UInt32 *timeStamp, Real64 *value, UInt16 *rawData)
{	Int32 error = MA200_SUCCESS;
	UInt16 data;

/*  Check parameters  */
	checkParamPointer(timeStamp, 2);
	checkParamPointer(value, 3);

/*	Read the fifo to get the first word  */
	checkErr(ma200_ReadRegister(handle, MA200_FIFO_DATA_REGISTER, &data));
	
/*  If data valid bit is set then read the fifo 2 more times  */	
	if (data & MA200_DATA_VALID_BIT)  //Check valid bit
	{	*timeStamp = (data << 16) & 0x7FFF0000l;
		checkErr(ma200_ReadRegister(handle, MA200_FIFO_DATA_REGISTER, &data));
		*timeStamp |= data;
		checkErr(ma200_ReadRegister(handle, MA200_FIFO_DATA_REGISTER, &data));
		*rawData = (data >> 4) & 0x0FFF;

	/* Calculate the real value  */		
  	switch(handle->input)
  	{	case MA200_UNIPOLAR_10VPP_INPUT:
				*value = (Real64)data/6552.0;									 
  			break;
  		case MA200_UNIPOLAR_20VPP_INPUT:
				*value = (Real64)data/3276.0;
  			break;
  		case MA200_BIPOLAR_10VPP_INPUT:
				*value = ((Real64)data-32760.0)/6552.0;
  			break;
  		case MA200_BIPOLAR_20VPP_INPUT:
				*value = ((Real64)data - 32760.0)/3276.0;
  			break;
  		default:
  			error = MA200_ERROR_UNINITIALIZED_INPUT_MODE;
  			break;
  	}
	}
	else
	{	error = MA200_ERROR_FIFO_EMPTY;
		goto Error;
	}

Error:
	return error;
}


/*****************************************************************************
*	ma200_ReadRawFifo 
*
*	Reads the next time stamp/data pair from the fifo.  If no valid data
*	exists in the fifo this function will return an error.
******************************************************************************/
Int32 ma200_ReadRawFifo (MA200_HANDLE handle, UInt32 *timeStamp, UInt16 *value)
{	Int32 error = MA200_SUCCESS;
	UInt16 data;

/*  Check parameters  */
	checkParamPointer(timeStamp, 2);
	checkParamPointer(value, 3);

/*	Read the fifo to get the first word  */
	checkErr(ma200_ReadRegister(handle, MA200_FIFO_DATA_REGISTER, &data));
	
/*  If data valid bit is set then read the fifo 2 more times  */	
	if (data & MA200_DATA_VALID_BIT)  //Check valid bit
	{	*timeStamp = (data << 16) & 0x7FFF0000l;
		checkErr(ma200_ReadRegister(handle, MA200_FIFO_DATA_REGISTER, &data));
		*timeStamp |= data;
		checkErr(ma200_ReadRegister(handle, MA200_FIFO_DATA_REGISTER, &data));
		*value = (data >> 4) & 0xFFF;
	}
	else
	{	error = MA200_ERROR_FIFO_EMPTY;
		goto Error;
	}

Error:
	return error;
}



/*****************************************************************************
*	chma200_ReadFifoBuffer
*
*	Reads a buffer of time stamp/data pairs from the fifo.  If no valid data
*	exists in the fifo this function will return an error.
******************************************************************************/
Int32 ma200_ReadFifoBuffer (MA200_HANDLE handle, Int32 count, UInt32 *timeStamp, Real64 *value, UInt16 *rawData, UInt32 *num_read)
{	Int32 error = MA200_SUCCESS;
	Int32 i;

/*  Check parameters  */
	checkParamPointer(timeStamp, 2);
	checkParamPointer(value, 3);

	checkErr(ma200_ReadRawFifoBuffer (handle, count, timeStamp, rawData, num_read));

/* Calculate the real value  */		
	for(i=0; i<(*num_read); i++)
	{
  	switch(handle->input)
  	{	case MA200_UNIPOLAR_10VPP_INPUT:
				value[i] = (Real64)rawData[i]/6552.0;
  			break;
  		case MA200_UNIPOLAR_20VPP_INPUT:
				value[i] = (Real64)rawData[i]/3276.0;
  			break;
  		case MA200_BIPOLAR_10VPP_INPUT:
				value[i] = ((Real64)rawData[i]-32760.0)/6552.0;
  			break;
  		case MA200_BIPOLAR_20VPP_INPUT:
				value[i] = ((Real64)rawData[i] - 32760.0)/3276.0;
  			break;
  		default:
  			error = MA200_ERROR_UNINITIALIZED_INPUT_MODE;
  			break;
  	}
	}

	if(*num_read != count)
		error = MA200_SPECIFIC_WARNING_NUM_SAMPLES_NOT_AVAILABLE;

Error:
	return error;
}



/*****************************************************************************
*	ma200_ResetFifo
*
*	Resets the fifo.  This will also clear any of the fifo interrupts pending
******************************************************************************/
Int32 ma200_ResetFifo (MA200_HANDLE handle)
{	Int32 error = MA200_SUCCESS;
	UInt16 value;

/*  Read the current value of the control/status register  */
	checkErr(ma200_ReadRegister(handle, MA200_CONTROL_STATUS_REGISTER, &value));
	
/*  Set the reset fifo bit and write to the register  */	
	checkErr(ma200_WriteRegister(handle, MA200_CONTROL_STATUS_REGISTER, (value | MA200_RESET_FIFO_BIT)));
Error:
	return error;
}


/*****************************************************************************
*	ma200_FifoUnreadCount 
*
*	Returns the value of the fifo unread count register
******************************************************************************/
Int32 ma200_FifoUnreadCount (MA200_HANDLE handle, UInt16 *unreadCount)
{	Int32 error = MA200_SUCCESS;

/*  Check parameters  */
	checkParamPointer(unreadCount, 2);

	checkErr(ma200_ReadRegister(handle, MA200_FIFO_UNREAD_COUNT_REGISTER, unreadCount));
Error:
	return error;
}


/*****************************************************************************
*	ma200_GetFifoStatus 
*
*	Returns the values of the three fifo status bits.  These bits are in
*	the control status register bits 13-15.
******************************************************************************/
Int32 ma200_GetFifoStatus (MA200_HANDLE handle, Bool *fifoFull, Bool *fifoHalfFull, Bool *dataStored)
{	Int32 error = MA200_SUCCESS;
	UInt16 value;
	
/*  Check parameters  */
	checkParamPointer(fifoFull, 2);
	checkParamPointer(fifoHalfFull, 3);
	checkParamPointer(dataStored, 4);
	
	*dataStored = MA200_FALSE;
	*fifoFull = MA200_FALSE;
	*fifoHalfFull = MA200_FALSE;
	
/*  Read the control/status register  */
	checkErr(ma200_ReadRegister(handle, MA200_CONTROL_STATUS_REGISTER, &value));
	
/*  Check each status bit  */	
	if(value & MA200_DATA_STORED_BIT)
		*dataStored = MA200_TRUE;
	if(value & MA200_FIFO_FULL_BIT)
		*fifoFull = MA200_TRUE;
	if(value & MA200_FIFO_HALF_FULL_BIT)
		*fifoHalfFull = MA200_TRUE;
Error:
	return error;
}


/*****************************************************************************
*	ma200_ResetTimeStamp 
*
*	Resets the time stamp.
******************************************************************************/
Int32 ma200_ResetTimeStamp (MA200_HANDLE handle)
{	Int32 error = MA200_SUCCESS;
	UInt16 value;

/*  Read the current value of the control/status register  */
	checkErr(ma200_ReadRegister(handle, MA200_CONTROL_STATUS_REGISTER, &value));

/*  Set time stamp reset bit and clear the fifo status bits  */
	value &= ~MA200_STATUS_BITS;
	value |= MA200_RESET_TIME_STAMP_BIT;
	
/*  Write the new value to the register  */	
	checkErr(ma200_WriteRegister(handle, MA200_CONTROL_STATUS_REGISTER, value));
Error:
	return error;
}


/*****************************************************************************
*	ma200_SetInputModeAndRange 
*
*	Sets the input mode and range attributes
******************************************************************************/
Int32 ma200_SetInputModeAndRange (MA200_HANDLE handle,  Bool inputMode,  Bool inputRange)
{	Int32 error = MA200_SUCCESS;
	
/*  Check parameters  */
	checkParamRange(inputMode, 0, 1, 2);
	checkParamRange(inputRange, 0, 1, 3);
	

/*  Set the attributes  */	
	if(inputMode == MA200_INPUT_MODE_UNIPOLAR)
	{	if(inputRange == MA200_INPUT_RANGE_10VPP)
		  handle->input = MA200_UNIPOLAR_10VPP_INPUT;
		else
		  handle->input = MA200_UNIPOLAR_20VPP_INPUT;
	}																											 
	else 
	{	if(inputRange == MA200_INPUT_RANGE_10VPP)
		  handle->input = MA200_BIPOLAR_10VPP_INPUT;
		else
		  handle->input = MA200_BIPOLAR_20VPP_INPUT;
	}
Error:
	return error;
}


/*****************************************************************************
*	ma200_SetAperture 
*
*	Set the aperture high and aperture low values.  Calculate the raw data
*	using the software driver attributes
******************************************************************************/
Int32 ma200_SetAperture (MA200_HANDLE handle, Real64 apertureHigh, Real64 apertureLow)
{	Int32 error = MA200_SUCCESS;
	UInt16 aperture_high_data, aperture_low_data;


/* Calculate the data value  */		
 	switch(handle->input)
 	{	case MA200_UNIPOLAR_10VPP_INPUT: case MA200_BIPOLAR_10VPP_INPUT:
		/*  Check parameters  */
			checkParamRange(apertureHigh, 0, 10, 2);
			checkParamRange(apertureLow, 0, 10, 3);
			aperture_high_data = (UInt16)(apertureHigh * 6552.0);
			aperture_low_data = (UInt16)(apertureLow * 6552.0);
 			break;
 		case MA200_UNIPOLAR_20VPP_INPUT: case MA200_BIPOLAR_20VPP_INPUT:
		/*  Check parameters  */
			checkParamRange(apertureHigh, 0, 20, 2);
			checkParamRange(apertureLow, 0, 20, 3);
			aperture_high_data = (UInt16)(apertureHigh * 3276.0);
			aperture_low_data = (UInt16)(apertureLow * 3276.0);
 			break;
 		default:
 			error = MA200_ERROR_UNINITIALIZED_INPUT_MODE;
 			break;
 	}
 	
/*  Write to the registers  */
	checkErr(ma200_WriteRegister(handle, MA200_APERTURE_HIGH_REGISTER, aperture_high_data));
	checkErr(ma200_WriteRegister(handle, MA200_APERTURE_LOW_REGISTER, aperture_low_data));
	
Error:
	return error;
}


/*****************************************************************************
*	ma200_SetRawAperture 
*
*	Set the aperture high and low values.  Take the raw data as inputs.
******************************************************************************/
Int32 ma200_SetRawAperture (MA200_HANDLE handle, UInt16 apertureHigh, UInt16 apertureLow)
{	Int32 error = MA200_SUCCESS;

/*  Check parameters  */
	checkParamRange(apertureHigh, 0, 0xFFF, 2);
	checkParamRange(apertureLow, 0, 0xFFF, 3);

	checkErr(ma200_WriteRegister(handle, MA200_APERTURE_HIGH_REGISTER, (apertureHigh << 4)));
	checkErr(ma200_WriteRegister(handle, MA200_APERTURE_LOW_REGISTER, (apertureLow << 4)));
	
Error:
	return error;
}


/*****************************************************************************
*	ma200_AccessoryModuleEnable
*
*	Enable the accessory module	
******************************************************************************/
Int32 ma200_AccessoryModuleEnable (MA200_HANDLE handle,  Bool enable)
{	Int32 error = MA200_SUCCESS;
	UInt16 value;

/*  Check parameters  */
	checkParamRange(enable, 0, 1, 2);

/*  Read the current value of the accessory module register  */
	checkErr(ma200_ReadRegister(handle, MA200_ACCESSORY_MODULE_REGISTER, &value));
	
/*  Set or clear the enable bit  */	
	if(enable == MA200_TRUE)
		value |= MA200_ACCESSORY_ENABLE_BIT;
	else
		value &= ~MA200_ACCESSORY_ENABLE_BIT;
		
/*  Write the new value to the accessory module register  */		
	checkErr(ma200_WriteRegister(handle, MA200_ACCESSORY_MODULE_REGISTER, value));

Error:
	return error;
}


/*****************************************************************************
*	ma200_SetCutOffFrequency 
*
* Set the cutoff frequency of the accesssory module filter
******************************************************************************/
Int32 ma200_SetCutOffFrequency (MA200_HANDLE handle, Int32 cutoffFrequency)
{	Int32 error = MA200_SUCCESS;
	UInt16 value;
	
/*  Check parameters  */
	checkParamRange(cutoffFrequency, 0, 7, 2);

/*  Read the current value of the accessory module register  */
	checkErr(ma200_ReadRegister(handle, MA200_ACCESSORY_MODULE_REGISTER, &value));

/*  Set the control bits  */
	value &= ~MA200_ACCESSORY_CONTROL_BITS;
	value |= cutoffFrequency;

/*  Write the new value to the accessory module register  */		
	checkErr(ma200_WriteRegister(handle, MA200_ACCESSORY_MODULE_REGISTER, value));
	
Error:
	return error;
}


/*****************************************************************************
*	ma200_CalibrationControl 
*
*	Control the trim potentionmeters using the calibration register
******************************************************************************/
Int32 ma200_CalibrationControl (MA200_HANDLE handle, Int32 controlFunction)
{	Int32 error = MA200_SUCCESS;
	UInt16 data = 0;

/*  Set the pot selection and the up/down bit  */
	switch(controlFunction)
	{	case MA200_CAL_FS_UP:
			data |= MA200_CALIBRATE_FULL_SCALE | MA200_CALIBRATE_UP_DOWN_BIT;			
			break;
		case MA200_CAL_FS_DOWN:
			data |= MA200_CALIBRATE_FULL_SCALE;			
			break;
		case MA200_CAL_OFFSET_UP:
			data |= MA200_CALIBRATE_OFFSET | MA200_CALIBRATE_UP_DOWN_BIT;			
			break;
		case MA200_CAL_OFFSET_DOWN:
			data |= MA200_CALIBRATE_OFFSET;			
			break;
		default:
			error = MA200_ERROR_PARAMETER2;
			goto Error;
			break;
	}
/*  Write to the calibration register  */	
	checkErr(ma200_WriteRegister(handle, MA200_CALIBRATION_REGISTER, data));
	
/*  Raise the clock  */
	checkErr(ma200_WriteRegister(handle, MA200_CALIBRATION_REGISTER, data | MA200_CALIBRATION_CLOCK_BIT));

/*  Lower the clock  */
	checkErr(ma200_WriteRegister(handle, MA200_CALIBRATION_REGISTER, data & ~MA200_CALIBRATION_CLOCK_BIT));
Error:
	return error;
}


/*****************************************************************************
*	ma200_ClockSpeedError 
*
*	Read the clock speed error bit
******************************************************************************/
Int32 ma200_ClockSpeedError (MA200_HANDLE handle, Bool *clockSpeedError)
{	Int32 error = MA200_SUCCESS;
	UInt16 value;

/*  Check parameters  */
	checkParamPointer(clockSpeedError, 2);

/*  Read the control/status register  */
	checkErr(ma200_ReadRegister(handle, MA200_CONTROL_STATUS_REGISTER, &value));
	
/*  Check the status bit  */	
	if(value & MA200_CLOCK_SPEED_ERROR_BIT)
		*clockSpeedError = MA200_TRUE;
	else
		*clockSpeedError = MA200_FALSE;
Error:
	return error;
}


/*****************************************************************************
*	ma200_IDWriteByte
*
*	Write a command byte to the ID PROM of the instrument.
*	The instruments ID PROM is serially accessed
*****************************************************************************/
static Int32 ma200_IDWriteByte(MA200_HANDLE handle, UInt8 value)
{	Int32 error = MA200_SUCCESS;
	Int32 i;
	
/*  Initialize  */	
	checkErr(ma200_WriteRegister(handle, MA200_ID_ACCESS_REGISTER, 0));
	checkErr(ma200_WriteRegister(handle, MA200_ID_ACCESS_REGISTER, 0x4));
	
/*  Write start bit  */	
	checkErr(ma200_IDWriteBit(handle, 1));
	for(i=0; i<=7; i++)
	{
	/*  Write data bits  */		
		checkErr(ma200_IDWriteBit(handle, ((value &0x80)>>7)));
		value = value << 1;
	}
Error:	
	return error;
}


/*****************************************************************************
*	ma200_IDWriteBit
*
*	Write a single bit to the ID PROM of the instrument.
*	The instruments ID PROM is serially accessed
*****************************************************************************/
static Int32 ma200_IDWriteBit(MA200_HANDLE handle,  Bool value)
{	Int32 error = MA200_SUCCESS;

/*  data with clock low  */
	checkErr(ma200_WriteRegister(handle, MA200_ID_ACCESS_REGISTER, (0x4 | (value & 0x1))));	

/*  data with clock high  */	 
	checkErr(ma200_WriteRegister(handle, MA200_ID_ACCESS_REGISTER, (0x6 | (value & 0x1))));	
Error:	
	return error;
}


/*****************************************************************************
*	ma200_IDReadByte
*
*	Reads a byte from the ID PROM of the instrument.
*	The instruments ID PROM is serially accessed
*****************************************************************************/
static Int32 ma200_IDReadByte(MA200_HANDLE handle, UInt8 *value)
{	Int32 error = MA200_SUCCESS;
	Int32 i;
	 Bool data;
	
	*value = 0;
	for(i=7; i>=0; i--)
	{ 
	/*  read data 1 bit at a time  */	
		checkErr(ma200_IDReadBit(handle, &data));
		*value |= ((data & 0x1) << i);
	}
	
	
Error:	
	return error;
}


/*****************************************************************************
*	ma200_IDReadBit
*
*	Reads a single bit from the ID PROM of the instrument.
*	The instruments ID PROM is serially accessed
*****************************************************************************/
static Int32 ma200_IDReadBit(MA200_HANDLE handle,  Bool *value)
{	Int32 error = MA200_SUCCESS;
	UInt16 data;

/*  clock low  */ 	
	checkErr(ma200_WriteRegister(handle, MA200_ID_ACCESS_REGISTER, 0x4));	
	
/*  clock high  */	
	checkErr(ma200_WriteRegister(handle, MA200_ID_ACCESS_REGISTER, 0x6));	
	
/*  Read data  */	
	checkErr(ma200_ReadRegister(handle, MA200_ID_ACCESS_REGISTER, &data));
	*value = ( Bool)data;

Error:	
	return error;
}


/*****************************************************************************
*	Modify the functions below this point to use the driver in an 
*	environment not supported by an APIS platform support module. All functions
*	above this point should not have to be modified.
*
*	In general, any call to an APIS function (apis_func()) will need to be
*	modified to communicate with the instrument or operating system being used.
*
******************************************************************************/
/*****************************************************************************
*	ma200_Init
*
*	Opens a session to the instrument specified by the index parameter.  
*	If IDQuery is true then this function verifies that the instrument is a 
*	ma200.  If reset is true then this function resets the instrument to it's 
*	default state.
******************************************************************************/
Int32 ma200_Init (Int32 instr,  Bool IDQuery,  Bool reset, MA200_HANDLE *handle)
{	Int32 error = MA200_SUCCESS;
	Int32 warning = MA200_SUCCESS;
  UInt16 value;

/*  Check parameters  */
	checkParamRange(IDQuery, 0, 1, 2);
	checkParamRange(reset, 0, 1, 3);
	checkParamPointer(handle, 4);
	
/*  Allocate the data structure  */
	(*handle) = 0;
	(*handle) = malloc(sizeof(MA200_INSTR));
	if((*handle) == NULL)
	{	error = MA200_ERROR_OUT_OF_MEMORY;
		goto Error;
	}

	(*handle)->apis_handle = 0;
	apisCheckErrInit(apis_open((APIS_PATH)instr, &(*handle)->apis_handle));
	
/*  Initialize Attributes  */
  (*handle)->input = MA200_BIPOLAR_10VPP_INPUT;

	
/* Perform ID Query */
	if(IDQuery == MA200_TRUE)
	{	checkErr(ma200_GetIdentWord (*handle, 17, &value));
	  
		value &= 0x0FFF;
		if (value != (UInt16)MA200_VXI_MANF_ID) 
		{	error = MA200_ERROR_FAIL_ID_QUERY;
 			goto Error;
	  }

		checkErr(ma200_GetIdentWord (*handle, 18, &value));
		
		value &= 0x0FFF;
		if (value != (UInt16)MA200_VXI_MODEL_CODE) 
		{	error = MA200_ERROR_FAIL_ID_QUERY;
			goto Error;
		} 
	}
/*-- End of ID Query --*/


/* Reset Instrument */
  if (reset == MA200_TRUE)
  {	checkErr(ma200_Reset (*handle));
  }

Error:
		
	if((error == MA200_SUCCESS) && (warning != MA200_SUCCESS))		// if no errors then return the warnings		
		error = warning;

	return error;
}


/*****************************************************************************
*	ma200_Close
*
*	Close the VISA session to the instrument.
******************************************************************************/
Int32 ma200_Close (MA200_HANDLE handle)
{	Int32 error = MA200_SUCCESS;
	if(handle->apis_handle != NULL)
		apisCheckErr(apis_close(handle->apis_handle));
	free(handle);
Error:
	return error;
}
	


/*****************************************************************************
*	ma200_Reset
*	
*	Reset the instrument to it's default state
******************************************************************************/
Int32 ma200_Reset (MA200_HANDLE handle)
{	
	Int32 error = 0;
	apisCheckErr(apis_reset(handle->apis_handle));
Error:
	return error;
}


/*****************************************************************************
*	ma200_ReadRegister
*
*	Read the specified offset from the module's I/O space and returns
*	the result.
******************************************************************************/
Int32 ma200_ReadRegister (MA200_HANDLE handle, UInt16 offset, UInt16 *value)
{	Int32 error = MA200_SUCCESS;
	
/*  Check parameters  */
	checkParamPointer(value, 3);

	apisCheckErr(apis_read(handle->apis_handle, 2, offset, value));
Error:
	return error;
}



/*****************************************************************************
*	ma200_WriteRegister
*
*	Writes to the specified offset in the module's I/O space
******************************************************************************/
Int32 ma200_WriteRegister (MA200_HANDLE handle, UInt16 offset, UInt16 value)
{	Int32 error = MA200_SUCCESS;
	
	apisCheckErr(apis_write(handle->apis_handle, 2, offset, value));
Error:
	return error;
}

/*****************************************************************************
*	ma200_InstallIrq
*	
*	Installs the interrupts handler to be called when an interrupt occurs
*	and configures the carrier to pass interrupts onto the system.
******************************************************************************/
Int32 ma200_InstallIrq (MA200_HANDLE handle, void *interruptHandler, Int32 vector, Int32 level, Int32 mode, void *var_ptr)
{	Int32 error = MA200_SUCCESS;

/*  Check parameters  */
	checkParamPointer(interruptHandler, 2);

	apisCheckErr(apis_irqinstall(handle->apis_handle, interruptHandler, vector, level, mode, var_ptr)); 

Error:
	return error;
}


/*****************************************************************************
*	ma200_UninstallIrq
*
*	Un-installs the interrupt handler for the instrument and disables
*	interrupt handling for the instrument
******************************************************************************/
Int32 ma200_UninstallIrq (MA200_HANDLE handle)
{	Int32 error = MA200_SUCCESS;

	apisCheckErr(apis_irqremove(handle->apis_handle));

Error:
	return error;
}


/*****************************************************************************
*	ma200_GetApisError
*
*	Returns the saved apis error code
******************************************************************************/
Int32 ma200_GetApisError (MA200_HANDLE handle, UInt32 *apis_error)
{	Int32 error = MA200_SUCCESS;
	*apis_error = handle->apis_error;
	return error;
}


/*****************************************************************************
*	chma200_ReadRawFifoBuffer
*
*	Reads a buffer of data from the fifo.  If no valid data
*	exists in the fifo this function returns an error.
******************************************************************************/
Int32 ma200_ReadRawFifoBuffer (MA200_HANDLE handle, Int32 count, UInt32 *timeStamp, UInt16 *rawData, UInt32 *num_read)
{	Int32 error = MA200_SUCCESS;
	UInt16 *fifo_data;
	Int32 i, word;
	UInt32 value32;
	
	fifo_data = malloc(count * sizeof(UInt16) * 3);
	if(fifo_data == NULL)
	{	error = MA200_ERROR_OUT_OF_MEMORY;
		goto Error;
	}

	*num_read = 0;
	
/*	Read the fifo data port */
	apisCheckErr(apis_readfifo(handle->apis_handle, 2, MA200_FIFO_DATA_REGISTER, count * 3, fifo_data));

	for(i=0; i<count; i++)
	{
		if (fifo_data[i*3] & MA200_DATA_VALID_BIT)  //Check valid bit
		{	timeStamp[i] = (fifo_data[i*3] << 16) & 0x7FFF0000l;
			timeStamp[i] |= fifo_data[(i*3) + 1];
			rawData[i] = fifo_data[(i*3) + 2];
			(*num_read)++;
		}
	}

	if(*num_read != count)
		error = MA200_SPECIFIC_WARNING_NUM_SAMPLES_NOT_AVAILABLE;
	

Error:
	if(fifo_data != NULL)
		free(fifo_data);
	return error;
}
